/*
 * Authors: David Trowbridge <trowbrds@cs.colorado.edu>
 *
 * Copyright (C) 2004 Novell, Inc (www.novell.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtkbox.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkstock.h>
#include <gtk/gtkvbox.h>

#include <libedataserver/e-source.h>
#include <libgnome/gnome-i18n.h>
#include <libgnomeui/libgnomeui.h>

#include "cal-properties.h"
#include "../e-cal-config.h"

struct _prop_data {
	void *object;
	GSList *widgets;
	GSList *properties;
	char *name;
	ECConfig *config;
};

static void
eccp_dialog_response (GtkWidget *dialog, int response, struct _prop_data *prop_data)
{
	if (response == GTK_RESPONSE_OK)
		if (e_config_page_check ((EConfig *) prop_data->config, "00.general"))
			e_config_commit ((EConfig *) prop_data->config);
		else
			return;
	else
		e_config_abort ((EConfig *) prop_data->config);
	gtk_widget_destroy (dialog);
}

static void
eccp_commit (EConfig *ec, GSList *items, void *data)
{
	ECConfigTargetSource *t = ec->target;
	ESource *source = t->source;
	struct _prop_data *prop_data = data;
	GSList *l, *widgets;

	widgets = prop_data->widgets;
	for (l = prop_data->properties; l; l = g_slist_next (l)) {
		if (strcmp (l->data, "name") == 0) {
			GtkWidget *entry = widgets->data;
			e_source_set_name (source, gtk_entry_get_text (GTK_ENTRY (entry)));
		}
		else if (strcmp (l->data, "color") == 0) {
			GtkWidget *picker = widgets->data;
			e_source_set_color (source, colorpicker_get_color (GNOME_COLOR_PICKER (picker)));
		}
		widgets = g_slist_next (widgets);
	}
}

static void
eccp_free (EConfig *ec, GSList *items, void *data)
{
	struct _prop_data *prop_data = data;

	g_slist_free (prop_data->properties);
	g_slist_free (prop_data->widgets);
	g_slist_free (items);

	g_free (prop_data);
}

static GtkWidget *
eccp_get_source_type (EConfig *ec, EConfigItem *item, struct _GtkWidget *parent, struct _GtkWidget *old, void *data)
{
	GtkWidget *label, *type;
	int row;
	ECConfigTargetSource *t = ec->target;
	ESource *source = t->source;
	ESourceGroup *group = e_source_peek_group (source);
	char *markup;

	if (old)
		return old;

	row = ((GtkTable *)parent)->nrows;

	label = gtk_label_new (_("Type:"));
	gtk_widget_show (label);
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_table_attach (GTK_TABLE (parent), label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);

	type = gtk_label_new ("");
	gtk_widget_show (type);
	markup = g_strdup_printf ("<span weight=\"bold\">%s</span>", e_source_group_peek_name (group));
	gtk_label_set_markup (GTK_LABEL (type), markup);
	gtk_misc_set_alignment (GTK_MISC (type), 0.0, 0.5);
	g_free (markup);
	gtk_table_attach (GTK_TABLE (parent), type, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	/* HACK */
	gtk_table_set_row_spacings (GTK_TABLE (parent), 6);
	gtk_table_set_col_spacings (GTK_TABLE (parent), 6);

	return type;
}

static GtkWidget *
eccp_get_source_name (EConfig *ec, EConfigItem *item, struct _GtkWidget *parent, struct _GtkWidget *old, void *data)
{
	GtkWidget *label, *entry;
	int row;
	ECConfigTargetSource *t = ec->target;
	ESource *source = t->source;
	struct _prop_data *prop_data = data;

	if (old)
		return old;

	row = ((GtkTable*)parent)->nrows;

	label = gtk_label_new_with_mnemonic (_("_Name:"));
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (parent), label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);

	entry = gtk_entry_new ();
	gtk_widget_show (entry);
	gtk_table_attach (GTK_TABLE (parent), entry, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);

	prop_data->properties = g_slist_append (prop_data->properties, "name");
	prop_data->widgets = g_slist_append (prop_data->widgets, entry);

	if (source)
		gtk_entry_set_text (GTK_ENTRY (entry), e_source_peek_name (source));

	return entry;
}

static void
colorpicker_set_color (GnomeColorPicker *color, guint32 rgb)
{
	gnome_color_picker_set_i8 (color, (rgb & 0xff0000) >> 16, (rgb & 0xff00) >> 8, rgb & 0xff, 0xff);
}

static guint32
colorpicker_get_color (GnomeColorPicker *color)
{
	guint8 r, g, b, a;
	guint32 rgb = 0;

	gnome_color_picker_get_i8 (color, &r, &g, &b, &a);

	rgb = r;
	rgb <<= 8;
	rgb |= g;
	rgb <<= 8;
	rgb |= b;

	return rgb;
}

static GtkWidget *
eccp_get_source_color (EConfig *ec, EConfigItem *item, struct _GtkWidget *parent, struct _GtkWidget *old, void *data)
{
	GtkWidget *label, *picker;
	int row;
	struct _prop_data *prop_data = data;
	ECConfigTargetSource *t = ec->target;
	ESource *source = t->source;

	static guint32 assigned_colors[] = {
		0xBECEDD, /* 190 206 221     Blue */
		0xE2F0EF, /* 226 240 239     Light Blue */
		0xC6E2B7, /* 198 226 183     Green */
		0xE2F0D3, /* 226 240 211     Light Green */
		0xE2D4B7, /* 226 212 183     Khaki */
		0xEAEAC1, /* 234 234 193     Light Khaki */
		0xF0B8B7, /* 240 184 183     Pink */
		0xFED4D3, /* 254 212 211     Light Pink */
		0xE2C6E1, /* 226 198 225     Purple */
		0xF0E2EF  /* 240 226 239     Light Purple */
	};
	GRand *rand = g_rand_new ();
	guint32 color;

	if (old)
		return old;

	row = ((GtkTable*)parent)->nrows;

	color = assigned_colors[g_rand_int_range (rand, 0, 9)];
	g_rand_free (rand);

	label = gtk_label_new_with_mnemonic (_("C_olor:"));
	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
	gtk_widget_show (label);
	gtk_table_attach (GTK_TABLE (parent), label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);

	picker = gnome_color_picker_new ();
	gtk_widget_show (picker);
	gtk_label_set_mnemonic_widget (GTK_WIDGET (label), picker);
	gtk_table_attach (GTK_TABLE (parent), picker, 1, 2, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);

	if (source)
		e_source_get_color (source, &color);
	else
		/* FIXME */;

	colorpicker_set_color (GNOME_COLOR_PICKER (picker), color);

	prop_data->properties = g_slist_append (prop_data->properties, "color");
	prop_data->widgets = g_slist_append (prop_data->widgets, picker);

	return picker;
}

static ECConfigItem eccp_items[] = {
	{ E_CONFIG_BOOK, "", NULL },
	{ E_CONFIG_PAGE,          "00.general", N_("General") },
	{ E_CONFIG_SECTION_TABLE, "00.general/00.source", NULL },
	{ E_CONFIG_ITEM_TABLE,    "00.general/00.source/00.type", NULL, eccp_get_source_type },
	{ E_CONFIG_ITEM_TABLE,    "00.general/00.source/10.name", NULL, eccp_get_source_name },
	{ E_CONFIG_ITEM_TABLE,    "00.general/00.source/20.color", NULL, eccp_get_source_color },
};

/**
 * ec_calendar_properties_show:
 * @parent: parent window for dialog (current unused)
 * @source: the ESource corresponding to the calendar
 *
 * Show calendar properties for @source.
 **/
void
ec_calendar_properties_show (struct _GtkWindow *parent, ESource *source)
{
	GtkWidget *dialog, *w;
	struct _prop_data *prop_data;
	ECConfig *ec;
	ECConfigTargetSource *target;
	GSList *l;
	int i;

	if (source == NULL)
		return;

	prop_data = g_malloc0 (sizeof (*prop_data));
	prop_data->object = source;

	dialog = gtk_dialog_new_with_buttons (_("Calendar Properties"), NULL,
	                                      GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR | GTK_DIALOG_MODAL,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OK, GTK_RESPONSE_OK,
					      NULL);
	gtk_widget_ensure_style (dialog);

	ec = ec_config_new (E_CONFIG_BOOK, "com.novell.evolution.calendar.calendarProperties");
	prop_data->config = ec;
	l = NULL;
	for (i = 0; i < sizeof (eccp_items) / sizeof (eccp_items[0]); i++)
		l = g_slist_prepend (l, &eccp_items[i]);
	e_config_add_items ((EConfig *) ec, l, eccp_commit, NULL, eccp_free, prop_data);

	target = ec_config_target_new_source (ec, source);
	e_config_set_target ((EConfig *) ec, (EConfigTarget *) target);
	w = e_config_create_widget ((EConfig *) ec);
	gtk_widget_show (w);

	gtk_box_pack_start ((GtkBox *) ((GtkDialog *) dialog)->vbox, w, TRUE, TRUE, 0);

	/* we do 'apply on ok' ... since instant apply may apply some very long running tasks */

	g_signal_connect (dialog, "response", G_CALLBACK (eccp_dialog_response), prop_data);
	gtk_widget_show (dialog);
}
